@@ -182,6 +182,13 @@ class Model(BasicFunction):
         t3 = dot(self.matrix[self.problem_size: 2 * self.problem_size, :], t2)
         return self.expanded_scaffer__(t3) + f_bias
 
+    def __calculate_weights__(self, z, xichma):
+        weight = 1
+        temp = sum(z ** 2)
+        if temp != 0:
+            weight = (1.0 / sqrt(temp)) * exp(-temp / (2 * self.problem_size * xichma ** 2))
+        return weight
+
     def F21(self, name="Composition Function 1", f_bias=700):
         xichma = array([10, 20, 30, 40, 50])
         lamda = array([1, 1e-6, 1e-26, 1e-6, 0.1])
@@ -191,23 +198,23 @@ class Model(BasicFunction):
 
         # g1: Rotated Rosenbrock’s Function f6’
         g1 = lamda[0] * self.F6(f_bias=0) + bias[0]
-        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))
+        w1 = self.__calculate_weights__(t1, xichma[0])
 
         # g2: Rotated Different Powers Function f5’
         g2 = lamda[1] * self.different_powers__(t2) + bias[1]
-        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))
+        w2 = self.__calculate_weights__(t1, xichma[1])
 
         # g3 Rotated Bent Cigar Function f3’
         g3 = lamda[2] * self.F3(f_bias=0) + bias[2]
-        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))
+        w3 = self.__calculate_weights__(t1, xichma[2])
 
         # g4: Rotated Discus Function f4’
         g4 = lamda[3] * self.F4(f_bias=0) + bias[3]
-        w4 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[3] ** 2))
+        w4 = self.__calculate_weights__(t1, xichma[3])
 
         # g5: Sphere Function f1
         g5 = lamda[4] * self.F1(f_bias=0) + bias[4]
-        w5 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[4] ** 2))
+        w5 = self.__calculate_weights__(t1, xichma[4])
 
         sw = sum([w1, w2, w3, w4, w5])
         result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
@@ -221,13 +228,13 @@ class Model(BasicFunction):
 
         # g1-3: Schwefel's Function f14’
         g1 = lamda[0] * self.F14(f_bias=0) + bias[0]
-        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))
+        w1 = self.__calculate_weights__(t1, xichma[0])
 
         g2 = lamda[1] * self.F14(f_bias=0) + bias[1]
-        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))
+        w2 = self.__calculate_weights__(t1, xichma[1])
 
         g3 = lamda[2] * self.F14(f_bias=0) + bias[2]
-        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))
+        w3 = self.__calculate_weights__(t1, xichma[2])
 
         sw = sum([w1, w2, w3])
         result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
@@ -241,13 +248,13 @@ class Model(BasicFunction):
 
         # g1-3: Schwefel's Function f15’
         g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
-        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))
+        w1 = self.__calculate_weights__(t1, xichma[0])
 
         g2 = lamda[1] * self.F15(f_bias=0) + bias[1]
-        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))
+        w2 = self.__calculate_weights__(t1, xichma[1])
 
         g3 = lamda[2] * self.F15(f_bias=0) + bias[2]
-        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))
+        w3 = self.__calculate_weights__(t1, xichma[2])
 
         sw = sum([w1, w2, w3])
         result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
@@ -261,13 +268,13 @@ class Model(BasicFunction):
 
         # g1-3: Schwefel's Function f15’, f12', f9'
         g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
-        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))
+        w1 = self.__calculate_weights__(t1, xichma[0])
 
         g2 = lamda[1] * self.F12(f_bias=0) + bias[1]
-        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))
+        w2 = self.__calculate_weights__(t1, xichma[1])
 
         g3 = lamda[2] * self.F9(f_bias=0) + bias[2]
-        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))
+        w3 = self.__calculate_weights__(t1, xichma[2])
 
         sw = sum([w1, w2, w3])
         result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
@@ -281,13 +288,13 @@ class Model(BasicFunction):
 
         # g1-3: Schwefel's Function f15’, f12', f9'
         g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
-        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))
+        w1 = self.__calculate_weights__(t1, xichma[0])
 
         g2 = lamda[1] * self.F12(f_bias=0) + bias[1]
-        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))
+        w2 = self.__calculate_weights__(t1, xichma[1])
 
         g3 = lamda[2] * self.F9(f_bias=0) + bias[2]
-        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))
+        w3 = self.__calculate_weights__(t1, xichma[2])
 
         sw = sum([w1, w2, w3])
         result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
@@ -301,23 +308,23 @@ class Model(BasicFunction):
 
         # g1: Rotated Schwefel's Function f15’
         g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
-        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))
+        w1 = self.__calculate_weights__(t1, xichma[0])
 
         # g2: Rotated Rastrigin’s Function f12’
         g2 = lamda[1] * self.F12(f_bias=0) + bias[1]
-        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))
+        w2 = self.__calculate_weights__(t1, xichma[1])
 
         # g3: Rotated High Conditioned Elliptic Function f2’
         g3 = lamda[2] * self.F2(f_bias=0) + bias[2]
-        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))
+        w3 = self.__calculate_weights__(t1, xichma[2])
 
         # g4: Rotated Weierstrass Function f9’
         g4 = lamda[3] * self.F9(f_bias=0) + bias[3]
-        w4 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[3] ** 2))
+        w4 = self.__calculate_weights__(t1, xichma[3])
 
         # g5: Rotated Griewank’s Function f10
         g5 = lamda[4] * self.F10(f_bias=0) + bias[4]
-        w5 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[4] ** 2))
+        w5 = self.__calculate_weights__(t1, xichma[4])
 
         sw = sum([w1, w2, w3, w4, w5])
         result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
@@ -331,23 +338,23 @@ class Model(BasicFunction):
 
         # g1: Rotated Schwefel's Function f15’
         g1 = lamda[0] * self.F10(f_bias=0) + bias[0]
-        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))
+        w1 = self.__calculate_weights__(t1, xichma[0])
 
         # g2: Rotated Rastrigin’s Function f12’
         g2 = lamda[1] * self.F12(f_bias=0) + bias[1]
-        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))
+        w2 = self.__calculate_weights__(t1, xichma[1])
 
         # g3: Rotated High Conditioned Elliptic Function f2’
         g3 = lamda[2] * self.F15(f_bias=0) + bias[2]
-        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))
+        w3 = self.__calculate_weights__(t1, xichma[2])
 
         # g4: Rotated Weierstrass Function f9’
         g4 = lamda[3] * self.F9(f_bias=0) + bias[3]
-        w4 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[3] ** 2))
+        w4 = self.__calculate_weights__(t1, xichma[3])
 
         # g5: Rotated Griewank’s Function f10
         g5 = lamda[4] * self.F1(f_bias=0) + bias[4]
-        w5 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[4] ** 2))
+        w5 = self.__calculate_weights__(t1, xichma[4])
 
         sw = sum([w1, w2, w3, w4, w5])
         result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
@@ -361,23 +368,23 @@ class Model(BasicFunction):
 
         # g1: Rotated Schwefel's Function f15’
         g1 = lamda[0] * self.F19(f_bias=0) + bias[0]
-        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))
+        w1 = self.__calculate_weights__(t1, xichma[0])
 
         # g2: Rotated Rastrigin’s Function f12’
         g2 = lamda[1] * self.F7(f_bias=0) + bias[1]
-        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))
+        w2 = self.__calculate_weights__(t1, xichma[1])
 
         # g3: Rotated High Conditioned Elliptic Function f2’
         g3 = lamda[2] * self.F15(f_bias=0) + bias[2]
-        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))
+        w3 = self.__calculate_weights__(t1, xichma[2])
 
         # g4: Rotated Weierstrass Function f9’
         g4 = lamda[3] * self.F20(f_bias=0) + bias[3]
-        w4 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[3] ** 2))
+        w4 = self.__calculate_weights__(t1, xichma[3])
 
         # g5: Rotated Griewank’s Function f10
         g5 = lamda[4] * self.F1(f_bias=0) + bias[4]
-        w5 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[4] ** 2))
+        w5 = self.__calculate_weights__(t1, xichma[4])
 
         sw = sum([w1, w2, w3, w4, w5])
         result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
